/*
 * (C) Copyright ParaSoft Corporation 2022.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 */

#include "InterfaceInfoIterator.h"
#include "../Logger.h"
#include "Environment.h"
#include "../MacBasedMachineId.h"

#include <iphlpapi.h>
#include <shlwapi.h>
#include <stdio.h>
#include <stdlib.h>

 // Link with Iphlpapi.lib, Shlwapi.lib and Ws2_32.lib
#pragma comment(lib, "IPHLPAPI.lib")
#pragma comment(lib, "SHLWAPI.lib")
#pragma comment(lib, "Ws2_32.lib")

#define MALLOC(x) HeapAlloc(GetProcessHeap(), 0, (x))
#define FREE(x) HeapFree(GetProcessHeap(), 0, (x))

static unsigned char* getCustomMac()
{
    const char* customMacEnv = Environment::getenv("PARASOFT_SUPPORT_NET_INTERFACES");
    unsigned char* customMac = 0;

    if (customMacEnv) {
        Logger::getLogger().debug("[DEBUG] Using physical address from environment variable: %s\n", customMacEnv);

        customMac = new unsigned char[6];
        customMac[0] = 0;
        customMac[1] = 0;
        customMac[2] = 0;
        customMac[3] = 0;
        customMac[4] = 0;
        customMac[5] = 0;

        int idx = 0;
        const char* it = customMacEnv;
        for (it = customMacEnv; *it; ++it) {
            if (*it == ':' || *it == '-') {
                ++idx;
                if (idx == 6) {
                    return customMac;
                }
            } else if (*it >= '0' && *it <= '9') {
                customMac[idx] = customMac[idx] * 16 + (*it - '0');
            } else if (*it >= 'a' && *it <= 'f') {
                customMac[idx] = customMac[idx] * 16 + (*it - 'a' + 10);
            } else if (*it >= 'A' && *it <= 'F') {
                customMac[idx] = customMac[idx] * 16 + (*it - 'A' + 10);
            }
        }
    }

    return customMac;
}

static PIP_ADAPTER_ADDRESSES getAdaptersAddresses()
{
    // Allocate a 15 KB buffer to start with.
    ULONG outBufLen = 15 * 1024;

    // Gathering adapters from machine using windows api
    for (int i = 0; i < 3; ++i) {
        PIP_ADAPTER_ADDRESSES pAddresses = (IP_ADAPTER_ADDRESSES*)MALLOC(outBufLen);
        if (pAddresses == NULL) {
            Error::setErrorMessage("Memory allocation failed for IP_ADAPTER_ADDRESSES struct");
            Logger::getLogger().debug("Memory allocation failed for IP_ADAPTER_ADDRESSES struct");
            return 0x0;
        }

        // The order in which adapters appear in the list returned by this function
        // can be controlled from the Network Connections folder: select the Advanced Settings menu item from the Advanced menu
        Logger::getLogger().debug("[DEBUG] Getting adapters addresses, size for Addresses: %d \n", outBufLen);
        DWORD dwRetVal = GetAdaptersAddresses(AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, NULL, pAddresses, &outBufLen);
        if (dwRetVal == ERROR_BUFFER_OVERFLOW) {
            Logger::getLogger().debug("[DEBUG] ERROR BUFFER OVERFLOW catched, size for Addresses: %d \n", outBufLen);
            FREE(pAddresses);
        } else if (dwRetVal == ERROR_SUCCESS) {
            return pAddresses;
        } else {
            Logger::getLogger().debug("[DEBUG] ERROR DETECTED: %d. Ending machine id calculation \n", dwRetVal);
            FREE(pAddresses);
            break;
        }
    };

    Error::setErrorMessage("Getting adapters addresses failed");
    return 0x0;
}

static bool hasKeyword(PIP_ADAPTER_ADDRESSES pAddress, const wchar_t* keyword)
{
    if ((pAddress->FriendlyName != NULL) && (StrStrIW(pAddress->FriendlyName, keyword) != NULL)) {
        return true;
    }
    if ((pAddress->Description != NULL) && (StrStrIW(pAddress->Description, keyword) != NULL)) {
        return true;
    }
    return false;
}

InterfaceInfoIterator::InterfaceInfoIterator()
{
    _customMac = getCustomMac();
    _pAddressIt = getAdaptersAddresses();
}

InterfaceInfoIterator::~InterfaceInfoIterator()
{
    if (_customMac != 0) {
        delete[] _customMac;
    }
}

bool InterfaceInfoIterator::hasNext()
{
    return _pAddressIt != NULL;
}

InterfaceInfo* InterfaceInfoIterator::next()
{
    InterfaceInfo* result = new InterfaceInfo();

    fillInterfaceInfo(result, _pAddressIt);

    _pAddressIt = _pAddressIt->Next;
    return result;
}

void InterfaceInfoIterator::fillInterfaceInfo(InterfaceInfo* result, PIP_ADAPTER_ADDRESSES pAddress)
{
    
    if (pAddress->FriendlyName != NULL) {
        result->wname = _wcsdup(pAddress->FriendlyName);
    }    
    if (pAddress->Description != NULL) {
        result->wdescription = _wcsdup(pAddress->Description);
    }
    bool isVirtual = hasKeyword(pAddress, L"virtual") || hasKeyword(pAddress, L"vmware");
    bool isWiFi = hasKeyword(pAddress, L"Wi-Fi") || (pAddress->IfType == IF_TYPE_IEEE80211);
    bool isBluetooth = hasKeyword(pAddress, L"Bluetooth");

    if (isVirtual) {
        result->quality = 3;
    } else if (isWiFi || isBluetooth) {
        result->quality = 1;
    } else if (pAddress->IfType == IF_TYPE_ETHERNET_CSMACD) {
        result->quality = 0;
    } else {
        result->quality = 2;
    }

    if (pAddress->PhysicalAddressLength == 6) {
        result->hasMac = true;
        result->mac[0] = pAddress->PhysicalAddress[0];
        result->mac[1] = pAddress->PhysicalAddress[1];
        result->mac[2] = pAddress->PhysicalAddress[2];
        result->mac[3] = pAddress->PhysicalAddress[3];
        result->mac[4] = pAddress->PhysicalAddress[4];
        result->mac[5] = pAddress->PhysicalAddress[5];

        if (_customMac != 0) {
            if (MacBasedMachineId::isMacEqual(result->mac, _customMac, 6)) {
                result->isCustom = true;
            }
        }
    }
}
